@@ -71,18 +71,18 @@ async def login(ctx, email, password):
         pass
     client = RoborockApiClient(email)
     user_data = await client.pass_login(password)
-    context.update(LoginData({"user_data": user_data, "email": email}))
+    context.update(LoginData(user_data=user_data, email=email))
 
 
 async def _discover(ctx):
     context: RoborockContext = ctx.obj
     login_data = context.login_data()
+    if not login_data:
+        raise Exception("You need to login first")
     client = RoborockApiClient(login_data.email)
     home_data = await client.get_home_data(login_data.user_data)
-    context.update(LoginData({**login_data, "home_data": home_data}))
-    click.echo(
-        f"Discovered devices {', '.join([device.name for device in home_data.devices + home_data.received_devices])}"
-    )
+    context.update(LoginData(**login_data.as_dict(), home_data=home_data))
+    click.echo(f"Discovered devices {', '.join([device.name for device in home_data.get_all_devices()])}")
 
 
 @click.command()
@@ -121,8 +121,7 @@ async def command(ctx, cmd, device_id, params):
         await _discover(ctx)
         login_data = context.login_data()
     home_data = login_data.home_data
-    devices = home_data.devices + home_data.received_devices
-    device = next((device for device in devices if device.duid == device_id), None)
+    device = next((device for device in home_data.get_all_devices() if device.duid == device_id))
     device_info = RoborockDeviceInfo(device=device)
     mqtt_client = RoborockMqttClient(login_data.user_data, device_info)
     await mqtt_client.send_command(cmd, params)
