@@ -7,20 +7,18 @@ from typing import Any, Optional
 
 from dacite import Config, from_dict
 
-from roborock.code_mappings import (
-    RoborockDockTypeCode,
-    RoborockDockWashTowelModeCode,
-    RoborockMopIntensityCode,
-    RoborockStateCode,
-)
-
 from .code_mappings import (
     RoborockDockDustCollectionModeCode,
     RoborockDockErrorCode,
+    RoborockDockTypeCode,
+    RoborockDockWashTowelModeCode,
     RoborockErrorCode,
     RoborockFanPowerCode,
+    RoborockMopIntensityCode,
     RoborockMopModeCode,
+    RoborockStateCode,
 )
+from .const import FILTER_REPLACE_TIME, MAIN_BRUSH_REPLACE_TIME, SENSOR_DIRTY_REPLACE_TIME, SIDE_BRUSH_REPLACE_TIME
 
 
 def camelize(s: str):
@@ -179,12 +177,20 @@ class HomeData(RoborockBase):
     received_devices: Optional[list[HomeDataDevice]] = None
     rooms: Optional[list[HomeDataRoom]] = None
 
+    def get_all_devices(self) -> list[HomeDataDevice]:
+        devices = []
+        if self.devices is not None:
+            devices += self.devices
+        if self.received_devices is not None:
+            devices += self.received_devices
+        return devices
+
 
 @dataclass
 class LoginData(RoborockBase):
     user_data: UserData
-    home_data: HomeData
-    email: Optional[str] = None
+    email: str
+    home_data: Optional[HomeData] = None
 
 
 @dataclass
@@ -279,6 +285,20 @@ class Consumable(RoborockBase):
     strainer_work_times: Optional[int] = None
     dust_collection_work_times: Optional[int] = None
     cleaning_brush_work_times: Optional[int] = None
+    main_brush_time_left: Optional[int] = None
+    side_brush_time_left: Optional[int] = None
+    filter_time_left: Optional[int] = None
+    sensor_time_left: Optional[int] = None
+
+    def __post_init__(self):
+        self.main_brush_time_left = (
+            MAIN_BRUSH_REPLACE_TIME - self.main_brush_work_time if self.main_brush_work_time else None
+        )
+        self.side_brush_time_left = (
+            SIDE_BRUSH_REPLACE_TIME - self.side_brush_work_time if self.side_brush_work_time else None
+        )
+        self.filter_time_left = FILTER_REPLACE_TIME - self.filter_work_time if self.filter_work_time else None
+        self.sensor_time_left = SENSOR_DIRTY_REPLACE_TIME - self.sensor_dirty_time if self.sensor_dirty_time else None
 
 
 @dataclass
