@@ -1,7 +1,6 @@
 from __future__ import annotations
 
 import binascii
-import hashlib
 import json
 import math
 import struct
@@ -12,22 +11,14 @@ from random import randint
 from Crypto.Cipher import AES
 from Crypto.Util.Padding import pad, unpad
 
-from roborock.exceptions import RoborockException
-from roborock.typing import RoborockCommand
+from .exceptions import RoborockException
+from .protocol import Utils
+from .roborock_typing import RoborockCommand
 
+SALT = "TXdfu$jyZ#TZHsg4".encode()
 
-def md5bin(message: str) -> bytes:
-    md5 = hashlib.md5()
-    md5.update(message.encode())
-    return md5.digest()
-
-
-def encode_timestamp(_timestamp: int) -> str:
-    hex_value = f"{_timestamp:x}".zfill(8)
-    return "".join(list(map(lambda idx: hex_value[idx], [5, 6, 3, 7, 1, 2, 0, 4])))
-
-
-salt = "TXdfu$jyZ#TZHsg4"
+AP_CONFIG = 1
+SOCK_DISCOVERY = 2
 
 
 @dataclass
@@ -52,7 +43,7 @@ class RoborockMessage:
 
     def get_method(self) -> RoborockCommand | None:
         protocol = self.protocol
-        if protocol in [4, 101, 102]:
+        if protocol in [4, 5, 101, 102]:
             payload = json.loads(self.payload.decode())
             for data_point_number, data_point in payload.get("dps").items():
                 if data_point_number in ["101", "102"]:
@@ -81,7 +72,7 @@ class RoborockParser:
         for roborock_message in roborock_messages:
             if len(roborock_message.prefix) not in [0, 4]:
                 raise RoborockException("Invalid prefix")
-            aes_key = md5bin(encode_timestamp(roborock_message.timestamp) + local_key + salt)
+            aes_key = Utils.md5(Utils.encode_timestamp(roborock_message.timestamp) + local_key.encode() + SALT)
             cipher = AES.new(aes_key, AES.MODE_ECB)
             payload = roborock_message.payload
             if payload:
@@ -111,7 +102,7 @@ class RoborockParser:
         prefix = b""
         original_index = index
         if len(msg) - index < 17:
-            ## broken message
+            # broken message
             return [], msg[original_index:]
 
         if msg[index + 4 : index + 7] == "1.0".encode():
@@ -119,7 +110,8 @@ class RoborockParser:
             index += 4
         elif msg[index : index + 3] != "1.0".encode():
             raise RoborockException(f"Unknown protocol version {msg[0:3]!r}")
-        if len(msg) - index in [17]:
+        message_size = len(msg) - index
+        if message_size == 17:
             [version, request_id, random, timestamp, protocol] = struct.unpack_from("!3sIIIH", msg, index)
             return [
                 RoborockMessage(
@@ -134,9 +126,10 @@ class RoborockParser:
             ], b""
 
         if len(msg) - index < 19:
-            ## broken message
+            # broken message
             return [], msg[original_index:]
 
+        _format = "!3sIIIHH"
         [
             version,
             request_id,
@@ -144,8 +137,9 @@ class RoborockParser:
             timestamp,
             protocol,
             payload_len,
-        ] = struct.unpack_from("!3sIIIHH", msg, index)
-        index += 19
+        ] = struct.unpack_from(_format, msg, index)
+        format_size = struct.calcsize(_format)
+        index += format_size
 
         if payload_len + index + 4 > len(msg):
             ## broken message
@@ -156,13 +150,13 @@ class RoborockParser:
             index += 2
         else:
             [payload, expected_crc32] = struct.unpack_from(f"!{payload_len}sI", msg, index)
-            crc32 = binascii.crc32(msg[index - 19 : index + payload_len])
+            crc32 = binascii.crc32(msg[index - format_size : index + payload_len])
             index += 4 + payload_len
             if crc32 != expected_crc32:
                 raise RoborockException(f"Wrong CRC32 {crc32}, expected {expected_crc32}")
 
         if payload:
-            aes_key = md5bin(encode_timestamp(timestamp) + local_key + salt)
+            aes_key = Utils.md5(Utils.encode_timestamp(timestamp) + local_key.encode() + SALT)
             decipher = AES.new(aes_key, AES.MODE_ECB)
             payload = unpad(decipher.decrypt(payload), AES.block_size)
 
